// ============================================================================
//
// Copyright (c) 2010-2016, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ============================================================================


#ifndef BOARD_H
#define BOARD_H

#include <unistd.h> 
#include <time.h> 

#include <wx/wx.h>
#include <wx/wxprec.h>
#include <wx/dcbuffer.h>


#ifndef MAINFRAME_CLASS_
class MainFrame;
#endif

#ifndef GRID_CLASS_
class Grid;
#endif

#ifndef SOLVERTHREAD_CLASS_
class SolverThread;
#endif

#ifndef INITVALUESHOLDER_CLASS_
class InitValuesHolder;
#endif

#ifndef GENERATOR_CLASS_
class Generator;
#endif


#ifndef BOARD_CLASS_
#define BOARD_CLASS_

class Board : public wxPanel
{

public:
    Board(MainFrame *parent);
    virtual ~Board();
    void initGrid();
    void generateGrid(int difficulty);
    void cleanGrid();
    void resetGrid ();
    void redraw (wxSize size);
    int getSize();

    MainFrame* getMainFrame();

    void initialize (InitValuesHolder* holder);
    InitValuesHolder* getGridValuesHolder ();

    void displayGrid (Grid* grid);
    void setStatusText (const wxString& text);

    void showError (const char* message);

    void start();
    void stop();
    void solve();
    void solveBacktrack();
    void resolvingFinished();

    void doRefresh ();
    void sleep ();

    void setGrid (Grid* grid);
	
    wxFont* getNormalFont();
    wxFont* getSmallFont();	

    DECLARE_EVENT_TABLE()

protected:
    void OnSize(wxSizeEvent& event);
    void OnPaint(wxPaintEvent& event);
    void OnLeftDown(wxMouseEvent& event);
    void OnKeyDown(wxKeyEvent& event);
    void OnTimer(wxCommandEvent& event);

private:
    MainFrame *parent;

    wxFont* normalFont;
    wxFont* smallFont;	
	
    int size;
    Generator* generator;
    Grid* grid;
    Grid* gridForDisplay;

    int getSizeFromUser();
    int getValueFromUser();
    wxString getInputFromUser(const char* label, const char* title, const char* defaultValue, bool zeroAllowed);

    wxLogStderr* log;

    SolverThread* solver;
};

inline wxFont* Board::getNormalFont()
{
    return this->normalFont;
}
 
inline wxFont* Board::getSmallFont()
{
    return this->smallFont;
}

#endif

#endif
