// ============================================================================
//
// Copyright (c) 2010-2016, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ============================================================================



#include "Element.h"

#include "Cell.h"
#include "Group.h"
#include "Board.h"

#include "Common.h"
#include "Group.h"


Element::Element (int size)
{
    this->size = size;
    this->valueSet = new bool[size];

    for (int i = 0; i < size; i++)
    {
        this->valueSet[i] = false;
    }
}

std::vector<int>* Element::getCellPossibilities (int cellNumber) 
{
    return cells[cellNumber]->getPossibilities();
}

std::vector<int>* Element::listCellsWithPossibility (int possibility)
{
    std::vector<int>* retList = new std::vector<int>();  
   
    for (int counter = 0; counter < size; counter++) 
    {
        if (cells[counter]->hasPossibility(possibility))
        {   
            retList->push_back(counter);
        }
    }
    
    return retList;
}

void Element::handleCellSinglePossibility(std::vector<Cell*>* changedCells, Solver* solver)
{
    for (int counter = 0; counter < size; counter++)
    {
        Cell* cell =  cells[counter];
        if (!cell->hasValue())
        {
            if (cell->handleCellSinglePossibility(solver))
            {
                changedCells->push_back(cell);
            }
        }
    }
}

Square* Element::getMatchingSquare (int cellNumber)
{
    return cells[cellNumber]->getSquare();
}

bool Element::discardCellPossibility (int cellNumber, int value)
{
    return cells[cellNumber]->discardPossibility(value);
}

bool Element::isSinglePossibility(Cell* cell, int value)
{
    for (int counter = 0; counter < size; counter++)
    {
        if (cells[counter] != cell && cells[counter]->isPossible (value))
        {
            return false;
        }
    }
    return true;
}

void Element::highlight (Solver* solver)
{     
    for (int counter = 0; counter < size; counter++) 
    {
        cells[counter]->setHighlight (true);
    }
    solver->doRefresh();    
    solver->sleep();     
    
    for (int counter = 0; counter < size; counter++) 
    {
        cells[counter]->setHighlight (false);
    }
    solver->doRefresh();    
}

bool Element::isSameGroupOrLess (std::vector<int>* possGroup, std::vector<int>* otherPossGroup)
{
    int groupSize = possGroup->size();
    int otherGroupSize = otherPossGroup->size();
    
    if (otherGroupSize > groupSize)
    {
        return false;
    }
    
    for (int otherCounter = 0; otherCounter < otherGroupSize; otherCounter++)
    {
        int otherValue = (*otherPossGroup)[otherCounter];
        bool found = false;
        for (int counter = 0; counter < groupSize; counter++)
        {
            int value = (*possGroup)[counter];
            if (value == otherValue)
            {
                found = true;
            }
        }
        
        if (!found)
        {
            return false;
        }
    }
    
    return true;
}

Group* Element::getCellsWithSamePoss (std::vector<int>* possGroup)
{
    Group* sameGroup = new Group();
    
    //search (group size) number of cells containing the same possibilities or less
    for (int j = 0; j < size; j++)
    {
        std::vector<int>* otherPossGroup = getCellPossibilities (j);
        
        if (otherPossGroup->size() >=2 && otherPossGroup->size() <= 4)
        {
            if (isSameGroupOrLess (possGroup, otherPossGroup))
            {
                sameGroup->addCell(cells[j], j);
            }
        }
    }
    
    return sameGroup;
}
