// ============================================================================
//
// Copyright (c) 2010-2016, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ============================================================================


#ifndef ELEMENT_H_
#define ELEMENT_H_


#include <list>
#include <vector>


#ifndef CELL_CLASS_
class Cell;
#endif

#ifndef SQUARE_CLASS_
class Square;
#endif

#ifndef GROUP_CLASS_
class Group;
#endif

#ifndef SOLVER_CLASS_
class Solver;
#endif


#ifndef ELEMENT_CLASS_
#define ELEMENT_CLASS_


class Element
{
    
protected:    
    Cell** cells;
    int size;
    bool* valueSet;
    
public:    
    Element (int size);
    
    virtual bool containsValue (int value);
    virtual const char* getName() = 0;
    
    void bookValue (int value);
    void freeValue (int value);

    Square* getMatchingSquare (int cellNumber);
    std::vector<int>* listCellsWithPossibility (int possibility);

    virtual Group* getCellsWithSamePoss (std::vector<int>* possGroup);
    virtual void highlight (Solver* solver);
    virtual void handleCellSinglePossibility(std::vector<Cell*>* changedCells, Solver* solver);
    virtual bool isSinglePossibility(Cell* cell, int value);
    virtual std::vector<int>* getCellPossibilities (int cellNumber);
    virtual bool discardCellPossibility (int cellNumber, int value);
    bool isSameGroupOrLess (std::vector<int>* possGroup, std::vector<int>* otherPossGroup);
};

// inlining a few critical simple methods


inline bool Element::containsValue (int value)
{
    return this->valueSet[value - 1];
}

inline void Element::bookValue (int value)
{
    this->valueSet[value - 1] = true;
}

inline void Element::freeValue (int value)
{
    this->valueSet[value - 1] = false;
}


#endif

#endif /*ELEMENT_H_*/
