// ============================================================================
//
// Copyright (c) 2010-2016, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ============================================================================


#include "FileReader.h"


using namespace std;


FileReader::FileReader(int size)
{
    this->size = size;
}

InitValuesHolder* FileReader::initFromFile (wxFrame* parent)
{
    // ask the user which file we must load...
    wxFileDialog fd(parent, wxT("Choose the XML file to load"), wxT(""), wxT(""),
                    wxT("XML files|*.xml|All files|*.*"), wxFD_OPEN);

    fd.Center();
    if (fd.ShowModal() == wxID_CANCEL)
    {
        return NULL;
    }

    wxString filePath = fd.GetPath();

    InitValuesHolder* holder = new InitValuesHolder(size);

    loadInitFile (filePath, holder);

    return holder;

}

bool FileReader::loadInitFile (wxString fileName, InitValuesHolder* holder)
{
    if (holder == NULL)
    {
        return false;
    }

    wxXmlDocument doc;
    if (!doc.Load(fileName))
    {
        return false;
    }

    // start processing the XML file
    if (doc.GetRoot()->GetName() != wxT("grid"))
    {
        return false;
    }

    wxXmlNode *child = doc.GetRoot()->GetChildren();
    while (child != NULL)
    {
        if (child->GetName() == wxT("cell"))
        {
            // process properties of <cell>
            wxString columnAsString =
                child->GetAttribute(wxT("column"),
                                  wxT("-1"));

            wxString rowAsString =
                child->GetAttribute(wxT("row"),
                                  wxT("-1"));

            wxString valueAsString =
                child->GetAttribute(wxT("value"),
                                  wxT("-1"));

            wxString typeAsString =
                child->GetAttribute(wxT("type"),
                                  wxT("-1"));

            //cout << "column " << columnAsString << " - row " << rowAsString << " - value " << valueAsString << endl;

            if (!columnAsString.IsNumber())
            {
#ifdef __WXDEBUG__
                wxLogMessage (std2wx("throwing SoldokuException : Column number is not a number"));
#endif
                throw new SoldokuException ((char*)"Column number is not a number");
            }
            if (!rowAsString.IsNumber())
            {
#ifdef __WXDEBUG__
                wxLogMessage (std2wx("throwing SoldokuException : Row number is not a number"));
#endif
                throw new SoldokuException ((char*)"Row number is not a number");
            }
            if (!valueAsString.IsNumber())
            {
#ifdef __WXDEBUG__
                wxLogMessage (std2wx("throwing SoldokuException : Cell value is not a number"));
#endif
                throw new SoldokuException ((char*)"Cell Value is not a number");
            }
            if (!typeAsString.IsEmpty() && !typeAsString.IsNull())
            {
                if (!typeAsString.IsNumber())
                {
#ifdef __WXDEBUG__
                    wxLogMessage (std2wx("throwing SoldokuException : Cell type is not a number"));
#endif
                    throw new SoldokuException ((char*)"Cell Type is not a number");
                }
            }

            int colNumber = atoi (columnAsString.mb_str(*wxConvCurrent));
            int rowNumber = atoi (rowAsString.mb_str(*wxConvCurrent));
            int value = atoi (valueAsString.mb_str(*wxConvCurrent));
            int typeAsInt = 0;
            CellType cellType = sdST_INPUT;
            if (!typeAsString.IsEmpty() && !typeAsString.IsNull())
            {
                typeAsInt = atoi (typeAsString.mb_str(*wxConvCurrent));
                switch (typeAsInt)
                {
                case 0:
                    cellType = sdST_INPUT;
                    break;
                case 1:
                    cellType = sdST_COMPUTED;
                    break;
                case 2:
                    cellType = sdST_GUESS;
                    break;
                case 3:
                    cellType = sdST_POSS;
                    break;
                case 4:
                    cellType = sdST_FIXED;
                    break;
                }
            }

            holder->setCellValue(
                    rowNumber, colNumber, value);

            holder->setCellType(
                    rowNumber, colNumber, cellType);

            wxXmlNode *possibilitiesChild = child->GetChildren();
            while (possibilitiesChild != NULL)
            {
                if (possibilitiesChild->GetName() == wxT("possibilities"))
                {
                    std::vector<int>* possibilities = new std::vector<int>();

                    wxXmlNode *possChild = possibilitiesChild->GetChildren();
                    while (possChild != NULL)
                    {
                        if (possChild->GetName() == wxT("poss"))
                        {
                            wxString valueAsString =
                                possChild->GetAttribute(wxT("value"),
                                                      wxT("-1"));
                            int value = atoi (valueAsString.mb_str(*wxConvCurrent));
                            possibilities->push_back(value);
                        }

                        possChild = possChild->GetNext();
                    }

                    holder->setCellPossibilities(rowNumber, colNumber, possibilities);
                }

                possibilitiesChild = possibilitiesChild->GetNext();
            }
        }

        child = child->GetNext();
    }

    return true;
}

FileReader::~FileReader()
{
}

