// ============================================================================
//
// Copyright (c) 2010-2016, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ============================================================================


#include "FileWriter.h"

#include "Common.h"
#include "InitValuesHolder.h"

using namespace std;


FileWriter::FileWriter()
{
}

void FileWriter::saveToFile (wxFrame* parent, InitValuesHolder* holder)
{

    // ask the user which file we must load...
    wxFileDialog fd(parent, wxT("Choose the XML file to save to"), wxT(""), wxT(""),
                    wxT("XML files|*.xml|All files|*.*"), wxFD_SAVE);

    fd.Center();
    if (fd.ShowModal() == wxID_CANCEL)
    {
        return;
    }

    wxString filePath = fd.GetPath();

    saveInitFile (filePath, holder);
}

bool FileWriter::saveInitFile (wxString fileName, InitValuesHolder* holder)
{

    wxXmlDocument doc;

    // 1. start generating the XML file
    wxXmlNode* rootNode = new wxXmlNode (wxXML_ELEMENT_NODE, std2wx("grid"));
    doc.SetRoot(rootNode);

    // 2. Set loaded values
    for (int rowCount = 0; rowCount < holder->getSize(); rowCount++)
    {
        for (int colCount = 0; colCount < holder->getSize(); colCount++)
        {
            int cellValue = holder->getCellValue(rowCount, colCount);
            CellType cellType = holder->getCellType(rowCount, colCount);

            wxXmlNode* cellNode = new wxXmlNode (wxXML_ELEMENT_NODE, std2wx("cell"));

            std::string rowAsString;
            std::stringstream rowConvStream;
            rowConvStream << rowCount;
            rowConvStream >> rowAsString;
            cellNode->AddAttribute (std2wx("row"), std2wx(rowAsString.c_str()));

            std::string colAsString;
            std::stringstream colConvStream;
            colConvStream << colCount;
            colConvStream >> colAsString;
            cellNode->AddAttribute(std2wx("column"), std2wx(colAsString.c_str()));

            std::string valueAsString;
            std::stringstream valConvStream;
            valConvStream << cellValue;
            valConvStream >> valueAsString;
            cellNode->AddAttribute(std2wx("value"), std2wx(valueAsString.c_str()));

            int cellTypeInt = 0;
            switch (cellType)
            {
            case sdST_INPUT :
                cellTypeInt = 0;
                break;
            case sdST_COMPUTED :
                cellTypeInt = 1;
                break;
            case sdST_GUESS :
                cellTypeInt = 2;
                break;
            case sdST_POSS :
                cellTypeInt = 3;
                break;
            case sdST_FIXED :
                cellTypeInt = 4;
                break;
            }
            std::string typeAsString;
            std::stringstream typeConvStream;
            typeConvStream << cellTypeInt;
            typeConvStream >> typeAsString;
            cellNode->AddAttribute(std2wx("type"), std2wx(typeAsString.c_str()));

            rootNode->AddChild(cellNode);

            std::vector<int>* possibilities = holder->getCellPossibilities(rowCount, colCount);
            if (possibilities != NULL && possibilities->size() > 0)
            {
                wxXmlNode* possibilitiesNode = new wxXmlNode (wxXML_ELEMENT_NODE, std2wx("possibilities"));

                for( std::vector<int>::iterator it = possibilities->begin(); // not listMyClass.begin()
                    it != possibilities->end(); // not listMyClass.end()
                    ++it)
                {
                    int i = (*it);

                    wxXmlNode* possNode = new wxXmlNode (wxXML_ELEMENT_NODE, std2wx("poss"));

                    std::string valueAsString;
                    std::stringstream valConvStream;
                    valConvStream << i;
                    valConvStream >> valueAsString;
                    possNode->AddAttribute(std2wx("value"), std2wx(valueAsString.c_str()));

                    possibilitiesNode->AddChild(possNode);
                }

                cellNode->AddChild(possibilitiesNode);
            }

#ifdef __WXDEBUG__
            wxLogMessage (std2wx("FileWriter::saveInitFile - Cell (%d, %d) : setting value : %d"), rowCount, colCount, cellValue);
#endif
        }
    }

    // 3. Save document
    if (!doc.Save(fileName))
    {
        return false;
    }

    return true;
}

FileWriter::~FileWriter()
{
}

