// ============================================================================
//
// Copyright (c) 2010-2016, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ============================================================================


#ifndef GRID_H_
#define GRID_H_

#include <wx/wx.h>
#include <wx/wxprec.h>
#include <wx/dcbuffer.h>

#include <math.h>
#include <algorithm>

#include <stdio.h>
#include <stdlib.h>
#include <vector>

#include "Cell.h"


#ifndef CELL_CLASS_
class Cell;
#endif

#ifndef ROW_CLASS_
class Row;
#endif

#ifndef COLUMN_CLASS_
class Column;
#endif

#ifndef SQUARE_CLASS_
class Square;
#endif

#ifndef GROUP_CLASS_
class Group;
#endif

#ifndef BOARD_CLASS_
class Board;
#endif

#ifndef SOLVERTHREAD_CLASS_
class SolverThread;
#endif

#ifndef SOLVER_CLASS_
class Solver;
#endif

#ifndef INITVALUESHOLDER_CLASS_
class InitValuesHolder;
#endif


#ifndef GRID_CLASS_
#define GRID_CLASS_

class Grid
{

public:
    Grid(int size);
    Grid (Grid* original, bool randomize = false);
    Grid(int size, int** initialValues);

    virtual ~Grid();

    int getSize();

    void draw (Board* board, wxDC* dc, wxSize size);
    Cell* getCell (int rowIdx, int colIdx);
    Cell* getCell (wxPoint position);

    std::list<Cell*>* listCellsWithoutValues();
    std::list<Cell*>* listCellsWithValues();
    std::vector<Cell*>* listCellsWithValue(int value);

    void initialize (InitValuesHolder* holder);
    InitValuesHolder* getGridValuesHolder ();
    void switchRows (int first, int second);
    void switchColumns (int first, int second);

    bool isSolutionFinished();

    void start(Solver* solver);
    void stop();
    bool isEasilySolveable();
    bool isSolveableBacktrack();
    int countSettedCells();

    void solvePossibleCells (Solver* solver);
    void solveUsingGuess (Solver* solver, int maxRecursiveCalls = 1000);

    Cell* getRemainingCellWithLessPossibilities();
    Cell* getRemainingCellWithMorePossibilities();

    void solve(Solver* solver);

    void reset ();
    void clean ();

    void printOnConsole();

private:

    int size;
    Cell*** cells;
    Row **rows;
    Column **columns;
    Square **squares;
    bool started;

    void init(int size);
    void destroy();

    bool solvePossiblities(Solver* solver);
    bool solveElementPossiblities(Element* element, Solver* solver);
    bool solveSinglePossibilities(Solver* solver);
    void solveSinglePossibilitiesInList(std::vector<Cell*>* changedCells, Solver* solver);
    void setPossiblities(Solver* solver);
    bool solveSquarePossiblities(Solver* solver);
    bool solveSquareByElementPossibilities(Element* element, Solver* solver);
    bool setRemainingPossibilities(Solver* solver);
    void resetPossibilities(Solver* solver);

};

// inlining a few critical methods

inline int Grid::getSize()
{
    return size;
}

inline Cell* Grid::getCell (int rowIdx, int colIdx)
{
    return cells[rowIdx][colIdx];
}

#endif


#endif
