// ============================================================================
//
// Copyright (c) 2010-2016, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ============================================================================


#include "InitValuesHolder.h"


#include "Cell.h"


InitValuesHolder::InitValuesHolder(int size)
{
    this->size = size;

    cellValues = new int*[size];
    cellTypes = new CellType*[size];
    cellPossibilities = new std::vector<int>**[size];

    // 1. Create array
    for (int count = 0; count < size; count++)
    {
        cellValues[count] = new int[size];
        cellTypes[count] = new CellType[size];
        cellPossibilities[count] = new std::vector<int>*[size];
    }

    // 2. Init array
    for (int rowCount = 0; rowCount < size; rowCount++)
    {
        for (int colCount = 0; colCount < size; colCount++)
        {
            cellValues[rowCount][colCount] = 0;
            cellTypes[rowCount][colCount] = sdST_INPUT;
            cellPossibilities[rowCount][colCount] = NULL;
        }
    }
}

InitValuesHolder::~InitValuesHolder()
{
    for (int rowCount = 0; rowCount < size; rowCount++)
    {
        delete[] cellValues[rowCount];
        delete[] cellTypes[rowCount];

        for (int colCount = 0; colCount < size; colCount++)
        {
            if (cellPossibilities[rowCount][colCount] != NULL)
            {
                delete cellPossibilities[rowCount][colCount];
            }
        }
        delete[] cellPossibilities[rowCount];
    }

    delete[] cellValues;
    delete[] cellTypes;
    delete[] cellPossibilities;
    cellValues = NULL;
}

int InitValuesHolder::getSize()
{
    return size;
}

void InitValuesHolder::setCellValue (int row, int column, int value)
{
    if (row >= size)
    {
#ifdef __WXDEBUG__
        wxLogMessage (std2wx("throwing SoldokuException : Asked row is above size"));
#endif
        throw new SoldokuException ((char*)"Asked row is above size");
    }
    if (column >= size)
    {
#ifdef __WXDEBUG__
        wxLogMessage (std2wx("throwing SoldokuException : Asked column is above size"));
#endif
        throw new SoldokuException ((char*)"Asked column is above size");
    }
    cellValues[row][column] = value;
}

int InitValuesHolder::getCellValue (int row, int column)
{
    if (row >= size)
    {
#ifdef __WXDEBUG__
        wxLogMessage (std2wx("throwing SoldokuException : Asked row is above size"));
#endif
        throw new SoldokuException ((char*)"Asked row is above size");
    }
    if (column >= size)
    {
#ifdef __WXDEBUG__
        wxLogMessage (std2wx("throwing SoldokuException : Asked column is above size"));
#endif
        throw new SoldokuException ((char*)"Asked column is above size");
    }

    return cellValues[row][column];
}

void InitValuesHolder::setCellType (int row, int column, CellType cellType)
{
    if (row >= size)
    {
#ifdef __WXDEBUG__
        wxLogMessage (std2wx("throwing SoldokuException : Asked row is above size"));
#endif
        throw new SoldokuException ((char*)"Asked row is above size");
    }
    if (column >= size)
    {
#ifdef __WXDEBUG__
        wxLogMessage (std2wx("throwing SoldokuException : Asked column is above size"));
#endif
        throw new SoldokuException ((char*)"Asked column is above size");
    }
    cellTypes[row][column] = cellType;
}

CellType InitValuesHolder::getCellType (int row, int column)
{
    if (row >= size)
    {
#ifdef __WXDEBUG__
        wxLogMessage (std2wx("throwing SoldokuException : Asked row is above size"));
#endif
        throw new SoldokuException ((char*)"Asked row is above size");
    }
    if (column >= size)
    {
#ifdef __WXDEBUG__
        wxLogMessage (std2wx("throwing SoldokuException : Asked column is above size"));
#endif
        throw new SoldokuException ((char*)"Asked column is above size");
    }

    return cellTypes[row][column];
}

void InitValuesHolder::setCellPossibilities (int row, int column, std::vector<int>* possibilities)
{
    if (row >= size)
    {
#ifdef __WXDEBUG__
        wxLogMessage (std2wx("throwing SoldokuException : Asked row is above size"));
#endif
        throw new SoldokuException ((char*)"Asked row is above size");
    }
    if (column >= size)
    {
#ifdef __WXDEBUG__
        wxLogMessage (std2wx("throwing SoldokuException : Asked column is above size"));
#endif
        throw new SoldokuException ((char*)"Asked column is above size");
    }
    cellPossibilities[row][column] = possibilities;
}


std::vector<int>* InitValuesHolder::getCellPossibilities (int row, int column)
{
    if (row >= size)
    {
#ifdef __WXDEBUG__
        wxLogMessage (std2wx("throwing SoldokuException : Asked row is above size"));
#endif
        throw new SoldokuException ((char*)"Asked row is above size");
    }
    if (column >= size)
    {
#ifdef __WXDEBUG__
        wxLogMessage (std2wx("throwing SoldokuException : Asked column is above size"));
#endif
        throw new SoldokuException ((char*)"Asked column is above size");
    }

    return cellPossibilities[row][column];
}
