// ============================================================================
//
// Copyright (c) 2010-2016, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ============================================================================



#include "MainFrame.h"


wxBEGIN_EVENT_TABLE(MainFrame, wxFrame)
    EVT_SIZE(MainFrame::OnSize)
    EVT_MAXIMIZE(MainFrame::OnMaximize)
    EVT_BUTTON(wxID_YES,      MainFrame::OnStartStop)
    EVT_BUTTON(wxID_YESTOALL, MainFrame::OnSolve)
    EVT_BUTTON(wxID_ADD,      MainFrame::OnSolveBacktrack)
    EVT_BUTTON(wxID_OPEN,     MainFrame::OnLoadGrid)
    EVT_BUTTON(wxID_SAVE,     MainFrame::OnSaveGrid)
    EVT_BUTTON(wxID_SETUP,    MainFrame::OnGenerateGrid)
    EVT_BUTTON(wxID_CLEAR,    MainFrame::OnCleanGrid)
    EVT_BUTTON(wxID_RESET,    MainFrame::OnResetGrid)
	EVT_COMMAND(100000, wxEVT_COMMAND_TEXT_UPDATED, MainFrame::onRefreshCommandEvent)
	EVT_COMMAND(100001, wxEVT_COMMAND_TEXT_UPDATED, MainFrame::onStatusCommandEvent)
	EVT_COMMAND(100002, wxEVT_COMMAND_TEXT_UPDATED, MainFrame::onResolvingFinishedCommandEvent)
wxEND_EVENT_TABLE();


MainFrame::MainFrame(const wxString& title)
          :wxFrame(NULL, wxID_ANY, title, wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE)
{
    started = false;

    progScreen = NULL;

    wxStatusBar *sb = CreateStatusBar();

    sb->SetStatusText(wxT("0"));

    this->SetMinSize(wxSize(600, 600));

/*
#ifdef __WXMSW__
    this->SetMinSize(wxSize(608, 610));
#else
    this->SetMinSize(wxSize(620, 600));
#endif
*/

    vbox = new wxBoxSizer(wxVERTICAL);

    // Panel on top
    hbox1 = new wxBoxSizer(wxHORIZONTAL);

    board = new Board(this);
    hbox1->Add(board, 1, wxEXPAND | wxALL);

    tb = new wxPanel (this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxEXPAND | wxRIGHT);

    wxSizer* tbSizer = new wxBoxSizer(wxHORIZONTAL);

    tbSizer->Add(new wxButton(tb, wxID_SETUP, wxT("Generate"), wxDefaultPosition, wxSize(70, -1)), 0, wxEXPAND | wxTOP, 2);

    launchButton = new wxButton(tb, wxID_YES, wxT("Launch ..."), wxDefaultPosition, wxSize(80, -1));
    tbSizer->Add(launchButton, 0, wxEXPAND | wxTOP, 2);

    tbSizer->Add(new wxButton(tb, wxID_YESTOALL, wxT("Solve!"), wxDefaultPosition, wxSize(60, -1)), 0, wxEXPAND | wxTOP, 2);
    tbSizer->Add(new wxButton(tb, wxID_ADD, wxT("Solve BT"), wxDefaultPosition, wxSize(70, -1)), 0, wxEXPAND | wxTOP, 2);
    tbSizer->Add(new wxButton(tb, wxID_OPEN, wxT("Load Grid"), wxDefaultPosition, wxSize(80, -1)), 0, wxEXPAND | wxTOP, 2);
    tbSizer->Add(new wxButton(tb, wxID_SAVE, wxT("Save Grid"), wxDefaultPosition, wxSize(80, -1)), 0, wxEXPAND | wxTOP, 2);
    tbSizer->Add(new wxButton(tb, wxID_RESET, wxT("Reset"), wxDefaultPosition, wxSize(60, -1)), 0, wxEXPAND | wxTOP, 2);
    tbSizer->Add(new wxButton(tb, wxID_CLEAR, wxT("Clean"), wxDefaultPosition, wxSize(60, -1)), 0, wxEXPAND | wxTOP, 2);

    tbSizer->Add(0, 0, 1, wxEXPAND | wxRIGHT | wxLEFT);

    // Put it all together
    vbox->Add(hbox1, 1, wxEXPAND | wxRIGHT);
    vbox->Add(tb, 0, wxEXPAND | wxRIGHT);

    tb->SetSizer(tbSizer);

    this->SetSizer(vbox);

    board->SetFocus();

    this->Centre();

    board->initGrid();
}

void MainFrame::wait() {
    // This is buggy under Wind32, hence the use of a messageDialog above
    progScreen = new wxProgressDialog(
            std2wx ("Please wait"),
            std2wx ("Wait for Sudoku generation ..."),
            100,
            this,
            wxPD_AUTO_HIDE);
    progScreen->Center();
    progScreen->Show();
}

void MainFrame::update() {
    progScreen->Pulse();
}

void MainFrame::stopWait() {
    progScreen->Hide();
    delete progScreen;
}

void MainFrame::setStatusText (const wxString& text)
{
    this->GetStatusBar()->SetStatusText(text);
}

void MainFrame::OnStartStop(wxCommandEvent& event)
{
    if (started)
    {
        board->stop();
        started = false;
        launchButton->SetLabel(wxT("Launch ..."));
    }
    else
    {
        started = true;
        launchButton->SetLabel(wxT("Stop!"));
        board->start();
    }
    Refresh();
}

void MainFrame::OnSolve(wxCommandEvent& event)
{
    board->solve();
    Refresh();
}

void MainFrame::OnSolveBacktrack(wxCommandEvent& event)
{
    board->solveBacktrack();
    Refresh();
}

void MainFrame::OnLoadGrid(wxCommandEvent& event)
{

    FileReader reader (board->getSize());

    InitValuesHolder* holder = reader.initFromFile(this);
    if (holder != NULL)
    {
        board->initialize (holder);
        delete holder;
    }

}

void MainFrame::OnSaveGrid(wxCommandEvent& event)
{
    FileWriter writer;

    InitValuesHolder* holder = board->getGridValuesHolder ();
    writer.saveToFile (this, holder);

    delete holder;
}

void MainFrame::OnGenerateGrid(wxCommandEvent& event)
{
    DifficultyChooser* dc = new DifficultyChooser(this);
    dc->Center();
    dc->ShowModal();

}
void MainFrame::generateGrid(int difficulty)
{
    this->wait();
    board->generateGrid(difficulty);
}

void MainFrame::OnCleanGrid(wxCommandEvent& event)
{
    board->cleanGrid();
}

void MainFrame::OnResetGrid(wxCommandEvent& event)
{
    board->resetGrid();
}


void MainFrame::OnMaximize(wxMaximizeEvent& event)
{
    int x = this->GetClientSize().GetX();                               // status bar
    int y = this->GetClientSize().GetY() - this->tb->GetSize().GetY() - 35;

#ifdef __WXDEBUG__
    wxLogMessage (std2wx("MainFrame::OnMaximize - size %d / %d "), x, y);
#endif

    board->redraw(wxSize (x, y));
}

void MainFrame::OnSize(wxSizeEvent& event)
{
    int x = this->GetClientSize().GetX();                               // status bar
    int y = this->GetClientSize().GetY() - this->tb->GetSize().GetY() - 35;

#ifdef __WXDEBUG__
    wxLogMessage (std2wx("MainFrame::OnSize - size %d / %d "), x, y);
#endif

    board->redraw(wxSize (x, y));
    this->Layout();
}

void MainFrame::doRefresh()
{
    this->Refresh();
    this->Update();
}

// this is called when the event from the thread is received
void  MainFrame::onRefreshCommandEvent(wxCommandEvent& evt)
{
	this->doRefresh();
}

// this is called when the event from the thread is received
void  MainFrame::onStatusCommandEvent(wxCommandEvent& evt)
{
	this->setStatusText (evt.GetString() );
}

void MainFrame::onResolvingFinishedCommandEvent(wxCommandEvent& evt)
{
    started = false;
    launchButton->SetLabel(wxT("Launch ..."));
}

MainFrame::~MainFrame()
{
}
