// ============================================================================
//
// Copyright (c) 2010-2016, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ============================================================================


#ifndef MAINFRAME_H_
#define MAINFRAME_H_

#include <wx/wx.h>
#include <wx/dialog.h>
#include <wx/msgdlg.h>
#include <wx/progdlg.h>
#include <wx/panel.h>

#include <wx/toolbar.h>
#include <wx/tbarbase.h>

#include "Board.h"
#include "FileReader.h"
#include "FileWriter.h"
#include "InitValuesHolder.h"
#include "DifficultyChooser.h"


#ifndef BOARD_CLASS_
class Board;
#endif

#ifndef FILEREADER_CLASS_
class FileReader;
#endif

#ifndef FILEWRITER_CLASS_
class FileWriter;
#endif

#ifndef INITVALUESHOLDER_CLASS_
class InitValuesHolder;
#endif


#ifndef MAINFRAME_CLASS_
#define MAINFRAME_CLASS_


class MainFrame : public wxFrame
{
public:
	MainFrame(const wxString& title);
	virtual ~MainFrame();

	void setStatusText (const wxString& text);

	void doRefresh();

	void wait();
	void update();
	void stopWait();

	void generateGrid(int difficulty);

	wxDECLARE_EVENT_TABLE();

protected:
    void OnSize(wxSizeEvent& event);
    void OnMaximize(wxMaximizeEvent& event);
    void OnStartStop(wxCommandEvent& event);
    void OnSolve(wxCommandEvent& event);
    void OnSolveBacktrack(wxCommandEvent& event);
    void OnLoadGrid(wxCommandEvent& event);
    void OnSaveGrid(wxCommandEvent& event);
    void OnGenerateGrid(wxCommandEvent& event);
    void OnCleanGrid(wxCommandEvent& event);
    void OnResetGrid(wxCommandEvent& event);

    void onRefreshCommandEvent(wxCommandEvent& evt);
    void onStatusCommandEvent(wxCommandEvent& evt);
    void onResolvingFinishedCommandEvent(wxCommandEvent& evt);

private:
    wxProgressDialog* progScreen;

    Board* board;
    wxBoxSizer* vbox;
    wxBoxSizer* hbox1;
    wxPanel* tb;

    bool started;
    wxButton* launchButton;
};

#endif


#endif /*MAINFRAME_H_*/
