// ============================================================================
//
// Copyright (c) 2010-2016, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ============================================================================


#ifndef SOLVER_H_
#define SOLVER_H_

#include <list>
#include <vector>

#include <stdlib.h>

#include <wx/wx.h>

#include "Grid.h"
#include "Board.h"
#include "Cell.h"


#ifndef SOLVER_CLASS_
#define SOLVER_CLASS_

class Solver 
{
    
public:
    Solver (bool fastResolution = true, Board* board = NULL);
    virtual ~Solver();
    
    virtual bool shouldCont (void);
    virtual bool isFastResolution();
    virtual void doRefresh ();
    virtual void sleep ();
    virtual void setStatusText (const wxString& text);
    
    virtual void displayGrid (Grid* grid);
    
    void showError (const char* message);
    void handleSolveError (const char* message);

    std::list<Grid*>* findSolutions(Grid* source);
    bool isSolvable(Grid* source);

protected:
    Board* board;    
    
private:

    void __findSolutions(std::list<Grid*>* retList, int idx, int ecSize, Cell** emptyCellsArray, Grid* source, Grid* original);
    void __isSolvable(int* count, int idx, int ecSize, Cell** emptyCellsArray, Grid* source, Grid* original);

    bool fastResolution;

};

#endif


#endif /*SOLVER_H_*/
