// ============================================================================
//
// Copyright (c) 2010-2016, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ============================================================================


#include "Square.h"

#include "Common.h"

#include "Cell.h"
#include "Grid.h"
#include "Solver.h"



Square::Square(int size, int rowPosition, int colPosition)
       :Element (size * 3)
{
    this->size = size;
    this->rowPosition = rowPosition;
    this->colPosition = colPosition;    
}

void Square::init (Grid* grid) 
{
    cells = new Cell*[(int) pow (size, 2)];
    
    int cellCounter = 0;
    for (int rowCounter = rowPosition; rowCounter < rowPosition + size; rowCounter++) 
    {
        for (int colCounter = colPosition; colCounter < colPosition + size; colCounter++) 
        {
            Cell* cell = grid->getCell(rowCounter, colCounter); 
            cell->setSquare (this);
            cells[cellCounter] = cell;
            
            cellCounter++;
        }
    }
}

const char* Square::getName()
{
    std::string nameStr;
    std::ostringstream oss;
    oss << "Square (" << (rowPosition + 1) << ", " << (colPosition + 1) << ")";
    nameStr = oss.str();
    
    return nameStr.c_str();
}

bool Square::discardPossibilityFromOther (int possibility, Element* element)
{
    bool changesMade = false;
    int cellCounter = 0;
    for (int rowCounter = rowPosition; rowCounter < rowPosition + size; rowCounter++) 
    {
        for (int colCounter = colPosition; colCounter < colPosition + size; colCounter++) 
        {
            Row* row = cells[cellCounter]->getRow();
            Column* column = cells[cellCounter]->getColumn();
            
            if (row != element && column != element)
            {
                if (cells[cellCounter]->discardPossibility(possibility))
                {
                    changesMade = true;
                }
            }
            
            cellCounter++;
        }
    }      
    return changesMade;
}

void Square::initPosition (wxCoord posX, wxCoord posY, wxCoord sizeX, wxCoord sizeY) 
{
    this->posX = posX;
    this->posY = posY;
    this->sizeX = sizeX;
    this->sizeY = sizeY;
}

void Square::draw (wxDC* dc) 
{
    dc->SetPen(*wxBLACK_PEN);    
    dc->SetBrush(*wxTRANSPARENT_BRUSH);
    
    dc->DrawRectangle(posX, posY, sizeX, sizeY);    
}

Square::~Square()
{
}

